@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,20 +15,27 @@
 #
 # END HEADER
 
-from __future__ import division, print_function, absolute_import
+from __future__ import absolute_import, division, print_function
 
 from enum import IntEnum
 
 import attr
 
-from hypothesis.errors import Frozen, StopTest, InvalidArgument
-from hypothesis.internal.compat import hbytes, hrange, text_type, \
-    bit_length, benchmark_time, int_from_bytes, unicode_safe_repr
-from hypothesis.internal.escalation import mark_for_escalation
+from hypothesis.errors import Frozen, InvalidArgument, StopTest
+from hypothesis.internal.compat import (
+    benchmark_time,
+    bit_length,
+    hbytes,
+    hrange,
+    int_from_bytes,
+    text_type,
+    unicode_safe_repr,
+)
 from hypothesis.internal.conjecture.utils import calc_label_from_name
+from hypothesis.internal.escalation import mark_for_escalation
 
-TOP_LABEL = calc_label_from_name('top')
-DRAW_BYTES_LABEL = calc_label_from_name('draw_bytes() in ConjectureData')
+TOP_LABEL = calc_label_from_name("top")
+DRAW_BYTES_LABEL = calc_label_from_name("draw_bytes() in ConjectureData")
 
 
 class Status(IntEnum):
@@ -38,14 +45,39 @@ class Status(IntEnum):
     INTERESTING = 3
 
     def __repr__(self):
-        return 'Status.%s' % (self.name,)
+        return "Status.%s" % (self.name,)
 
 
 @attr.s(slots=True)
 class Example(object):
+    """Examples track the hierarchical structure of draws from the byte stream,
+    within a single test run.
+
+    Examples are created to mark regions of the byte stream that might be
+    useful to the shrinker, such as:
+    - The bytes used by a single draw from a strategy.
+    - Useful groupings within a strategy, such as individual list elements.
+    - Strategy-like helper functions that aren't first-class strategies.
+    - Each lowest-level draw of bits or bytes from the byte stream.
+    - A single top-level example that spans the entire input.
+
+    Example-tracking allows the shrinker to try "high-level" transformations,
+    such as rearranging or deleting the elements of a list, without having
+    to understand their exact representation in the byte stream.
+    """
+
+    # Depth of this example in the example tree. The top-level example has a
+    # depth of 0.
     depth = attr.ib()
+
+    # A label is an opaque value that associates each example with its
+    # approximate origin, such as a particular strategy class or a particular
+    # kind of draw.
     label = attr.ib()
+
+    # Index of this example inside the overall list of examples.
     index = attr.ib()
+
     start = attr.ib()
     end = attr.ib(default=None)
 
@@ -53,7 +85,14 @@ class Example(object):
     # All examples start out as trivial, and then get marked non-trivial when
     # we see a byte that is neither forced nor zero.
     trivial = attr.ib(default=True)
+
+    # True if we believe that the shrinker should be able to delete this
+    # example completely, without affecting the value produced by its enclosing
+    # strategy. Typically set when a rejection sampler decides to reject a
+    # generated value and try again.
     discarded = attr.ib(default=None)
+
+    # List of child examples, represented as indices into the example list.
     children = attr.ib(default=attr.Factory(list))
 
     @property
@@ -63,11 +102,27 @@ class Example(object):
 
 @attr.s(slots=True, frozen=True)
 class Block(object):
+    """Blocks track the flat list of lowest-level draws from the byte stream,
+    within a single test run.
+
+    Block-tracking allows the shrinker to try "low-level"
+    transformations, such as minimizing the numeric value of an
+    individual call to ``draw_bits``.
+    """
+
     start = attr.ib()
     end = attr.ib()
+
+    # Index of this block inside the overall list of blocks.
     index = attr.ib()
 
+    # True if this block's byte values were forced by a write operation.
+    # As long as the bytes before this block remain the same, modifying this
+    # block's bytes will have no effect.
     forced = attr.ib()
+
+    # True if this block's byte values are all 0. Reading this flag can be
+    # more convenient than explicitly checking a slice for non-zero bytes.
     all_zero = attr.ib()
 
     @property
@@ -90,14 +145,12 @@ MAX_DEPTH = 100
 
 
 class ConjectureData(object):
-
     @classmethod
     def for_buffer(self, buffer):
         buffer = hbytes(buffer)
         return ConjectureData(
             max_length=len(buffer),
-            draw_bytes=lambda data, n:
-            hbytes(buffer[data.index:data.index + n])
+            draw_bytes=lambda data, n: hbytes(buffer[data.index : data.index + n]),
         )
 
     def __init__(self, max_length, draw_bytes):
@@ -108,7 +161,7 @@ class ConjectureData(object):
         self.block_starts = {}
         self.blocks = []
         self.buffer = bytearray()
-        self.output = u''
+        self.output = u""
         self.status = Status.VALID
         self.frozen = False
         global global_test_counter
@@ -126,13 +179,12 @@ class ConjectureData(object):
         self.example_stack = []
         self.has_discards = False
 
-        self.start_example(TOP_LABEL)
+        top = self.start_example(TOP_LABEL)
+        assert top.depth == 0
 
     def __assert_not_frozen(self, name):
         if self.frozen:
-            raise Frozen(
-                'Cannot call %s on frozen ConjectureData' % (
-                    name,))
+            raise Frozen("Cannot call %s on frozen ConjectureData" % (name,))
 
     @property
     def depth(self):
@@ -148,17 +200,20 @@ class ConjectureData(object):
         return [block.bounds for block in self.blocks]
 
     def note(self, value):
-        self.__assert_not_frozen('note')
+        self.__assert_not_frozen("note")
         if not isinstance(value, text_type):
             value = unicode_safe_repr(value)
         self.output += value
 
     def draw(self, strategy, label=None):
         if self.is_find and not strategy.supports_find:
-            raise InvalidArgument((
-                'Cannot use strategy %r within a call to find (presumably '
-                'because it would be invalid after the call had ended).'
-            ) % (strategy,))
+            raise InvalidArgument(
+                (
+                    "Cannot use strategy %r within a call to find (presumably "
+                    "because it would be invalid after the call had ended)."
+                )
+                % (strategy,)
+            )
 
         if strategy.is_empty:
             self.mark_invalid()
@@ -191,14 +246,11 @@ class ConjectureData(object):
             self.stop_example()
 
     def start_example(self, label):
-        self.__assert_not_frozen('start_example')
+        self.__assert_not_frozen("start_example")
 
         i = len(self.examples)
         new_depth = self.depth + 1
-        ex = Example(
-            index=i,
-            depth=new_depth, label=label, start=self.index,
-        )
+        ex = Example(index=i, depth=new_depth, label=label, start=self.index)
         self.examples.append(ex)
         if self.example_stack:
             p = self.example_stack[-1]
@@ -263,7 +315,7 @@ class ConjectureData(object):
         del self._draw_bytes
 
     def draw_bits(self, n):
-        self.__assert_not_frozen('draw_bits')
+        self.__assert_not_frozen("draw_bits")
         if n == 0:
             result = 0
         elif n % 8 == 0:
@@ -284,7 +336,7 @@ class ConjectureData(object):
         return result
 
     def write(self, string):
-        self.__assert_not_frozen('write')
+        self.__assert_not_frozen("write")
         self.__check_capacity(len(string))
         assert isinstance(string, hbytes)
         original = self.index
@@ -322,9 +374,9 @@ class ConjectureData(object):
         self.stop_example()
 
     def draw_bytes(self, n):
-        self.__assert_not_frozen('draw_bytes')
+        self.__assert_not_frozen("draw_bytes")
         if n == 0:
-            return hbytes(b'')
+            return hbytes(b"")
         self.__check_capacity(n)
         result = self._draw_bytes(self, n)
         assert len(result) == n
@@ -332,14 +384,14 @@ class ConjectureData(object):
         return hbytes(result)
 
     def mark_interesting(self, interesting_origin=None):
-        self.__assert_not_frozen('mark_interesting')
+        self.__assert_not_frozen("mark_interesting")
         self.interesting_origin = interesting_origin
         self.status = Status.INTERESTING
         self.freeze()
         raise StopTest(self.testcounter)
 
     def mark_invalid(self):
-        self.__assert_not_frozen('mark_invalid')
+        self.__assert_not_frozen("mark_invalid")
         self.status = Status.INVALID
         self.freeze()
         raise StopTest(self.testcounter)
